package cmdTemplate

import (
    "errors"
    "regexp"
    "strings"
    "encoding/json"
    "nexus-client/cmd"
    t "nexus-common/template"
    u "nexus-client/utils"
    g "nexus-client/globals"
    "github.com/google/uuid"
    "github.com/go-resty/resty/v2"
)

func printUsage(c cmd.Command, action string) {
    for _, desc := range c.GetDesc() {
        u.PrintlnErr(desc)
    }
    u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
    u.PrintlnErr("USAGE: ",c.GetName()," [ID ...] [regex ...]")
    u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
    u.PrintlnErr("Only templates matching the specified IDs or regexes will be "+action+".")
    const usage string = `Any number of IDs or regexes can be specified.
The regex only matches the templates's name and is case-insensitive.
Regex examples:
"."   -> matches any templates
"bla" -> matches any templates containing "bla"`
    u.PrintlnErr(usage)
}

func printRegexUsageDetails() {
    u.PrintlnErr("The action only applies to templates matching the specified IDs or regexes.")
    const usage string = `Any number of IDs or regexes can be specified.
The regex only matches the template's name and is case-insensitive.
Regex examples:
"."   -> matches any templates
"bla" -> matches any templates containing "bla" in their name`
    u.PrintlnErr(usage)
}

// Returns a list of filtered templates for a given route.
// Filters are based on patterns describing IDs or regexes.
// Patterns can either be:
// - any number of "template IDs" (UUID)
// - any number of "template names"
// - any combination of "template IDs" and "template names"
// - any regular expression (it applies to the template's name only)
// Remark: the matching is case-insensitive
// Regular expression examples:
//   "."   -> matches everything
//   "bla" -> matches any template name containing "bla"
func getFilteredTemplates(route string, patterns []string) ([]t.TemplateSerialized, error) {
    if len(patterns) < 1 {
        return nil, errors.New("At least one ID or regex must be specified")
    }

    client := g.GetInstance().Client
    host := g.GetInstance().Host

    var ids []string
    var regexes []string

    for _, pattern := range patterns {
        _, err := uuid.Parse(pattern)
        if err != nil {
            regexes = append(regexes, pattern)
        } else {
            ids = append(ids, pattern)
        }
    }

    resp, err := client.R().Get(host+route)
    if err != nil {
        return nil, err
    }

    templatesList := []t.TemplateSerialized{}

    if resp.IsSuccess() {
        templates, err := deserializeTemplates(resp)
        if err != nil {
            return nil, err
        }

        for _, template := range templates {
            found := false
            for _, id := range ids {
                if id == template.ID.String() {
                    templatesList = append(templatesList, template)
                    found = true
                    break
                }
            }
            if found {
                continue
            }
            for _, regex := range regexes {
                match, err := regexp.MatchString(strings.ToLower(regex), strings.ToLower(template.Name))
                if err != nil {
                    return nil, errors.New("Error matching \""+regex+"\": "+err.Error())
                } else {
                    if match {
                        templatesList = append(templatesList, template)
                        break
                    }
                }
            }
        }
        return templatesList, nil
    } else {
        return nil, errors.New("Error: "+resp.Status()+": "+resp.String())
    }
}

// Deserialize a list of templates from an http response (no filtering).
func deserializeTemplates(resp *resty.Response) ([]t.TemplateSerialized, error) {
    templates := []t.TemplateSerialized{}
    if err := json.Unmarshal(resp.Body(), &templates); err != nil {
        return nil, err
    }
    return templates, nil
}

// Deserialize a single template from an http response.
func deserializeTemplate(resp *resty.Response) (*t.TemplateSerialized, error) {
    var tpl *t.TemplateSerialized = &t.TemplateSerialized{}
    if err := json.Unmarshal(resp.Body(), &tpl); err != nil {
        return tpl, err
    }
    return tpl, nil
}
